--
-- KroneBigM500
-- Specialization for KroneBigM500
--
-- @author  	fruktor (Modding-Society)
-- @version 	v0.1
-- @date  		22/02/11
-- @history:	v0.1 - Initial version
--
--

KroneBigM500 = {};

function KroneBigM500.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Motorized, specializations);
end;

function KroneBigM500:load(xmlFile)

	self.setMowerOptions = SpecializationUtil.callSpecializationsFunction("setMowerOptions");
	self.setActiveMower = SpecializationUtil.callSpecializationsFunction("setActiveMower");
	--self.setBeaconLightsVisibility = SpecializationUtil.callSpecializationsFunction("setBeaconLightsVisibility");
	self.setCuttersState = SpecializationUtil.callSpecializationsFunction("setCuttersState");
	self.setVehicleRpmUp = SpecializationUtil.callSpecializationsFunction("setVehicleRpmUp");
	self.saveMinRpm = 0;

	
	--### create variables for mowers and set their states as in i3d
	self.mowers = {};
	for i=1, 4 do 
		self.mowers[i] = {};
		self.mowers[i].setUnfold		= true;
		self.mowers[i].setDown			= false;
		self.mowers[i].setTurnedOn		= false;
		self.mowers[i].setSwathOn		= true;
		self.mowers[i].isTransport		= false;
		self.mowers[i].isWorking		= true;
		self.mowers[i].isUp				= false;
		self.mowers[i].isDown			= false;
	end;
	
	self.mowerTypes = {};
	self.mowerTypes[1] = "front";
	self.mowerTypes[2] = "left";
	self.mowerTypes[3] = "right";
	self.mowerTypes[4] = "all";
	self.activeMowerIdx = 1;


	--### some information for automatic movement of the front mower 
	
	self.mowers[1].jointRotLimit = Utils.degToRad(20);
	self.mowers[1].jointRotLimit2 = Utils.degToRad(8);
	
	self.frontMower = {};

	self.frontMower.height = 0.84;
	local node1 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.kroneBigM.front#idx1"));
	self.frontMower.node1 = node1;
	local node2 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.kroneBigM.front#idx2"));
	self.frontMower.node2 = node2;
	
	local shutter1 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.kroneBigM.front#shutterIdx1"));
	self.mowers[1].shutter1 = shutter1;
	local shutter2 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.kroneBigM.front#shutterIdx2"));
	self.mowers[1].shutter2 = shutter2;
	self.mowers[1].shutterIsOpen = false;
	self.mowers[1].shutterIsClosed = true;

	local roll1 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.kroneBigM.front#rollIdx1"));
	self.mowers[1].roll1 = roll1;
	local roll2 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.kroneBigM.front#rollIdx2"));
	self.mowers[1].roll2 = roll2;
	
	local hydraulicIdx = getXMLString(xmlFile, "vehicle.kroneBigM.front#hydraulicIdx");
	local hydraulicNode = Utils.indexToObject(self.components, hydraulicIdx);
	self.frontMower.hydraulicNode = hydraulicNode;

	self.frontMower.moveAuto = false;
	
	local idx1 = getXMLInt(xmlFile, "vehicle.kroneBigM.front#jointIdx1");
	local idx2 = getXMLInt(xmlFile, "vehicle.kroneBigM.front#jointIdx2");
	self.mowers[1].idx1 = idx1;
	self.mowers[1].idx2 = idx2;
	local jointRefNode1 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.kroneBigM.front#jointRefIdx1"));
	local jointRefNode2 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.kroneBigM.front#jointRefIdx2"));
	self.mowers[1].jointRefNode1 = jointRefNode1;
	self.mowers[1].jointRefNode2 = jointRefNode2;

	
	--### now load the necessary informtion for the left and right mowers
	self.leftMower = {};
	local idx1 = getXMLInt(xmlFile, "vehicle.kroneBigM.left#jointIdx1");
	local idx2 = getXMLInt(xmlFile, "vehicle.kroneBigM.left#jointIdx2");
	local idx3 = getXMLInt(xmlFile, "vehicle.kroneBigM.left#jointIdx3");
	self.mowers[2].idx1 = idx1;
	self.mowers[2].idx2 = idx2;
	self.mowers[2].idx3 = idx3;
	self.mowers[2].jointRotLimits = {};
	self.mowers[2].jointRotLimits[1] = Utils.degToRad(10);
	self.mowers[2].jointRotLimits[2] = Utils.degToRad(8);
	self.mowers[2].jointRotLimits[3] = Utils.degToRad(8);
	self.mowers[2].rotationNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.kroneBigM.left#rotateIdx"));
	local shutter1 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.kroneBigM.left#shutterIdx"));
	self.mowers[2].shutter1 = shutter1;	
	self.mowers[2].shutterIsOpen = false;
	self.mowers[2].shutterIsClosed = true;
	local roll1 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.kroneBigM.left#rollIdx"));
	self.mowers[2].roll1 = roll1;
	
	self.rightMower = {};
	local idx1 = getXMLInt(xmlFile, "vehicle.kroneBigM.right#jointIdx1");
	local idx2 = getXMLInt(xmlFile, "vehicle.kroneBigM.right#jointIdx2");
	local idx3 = getXMLInt(xmlFile, "vehicle.kroneBigM.right#jointIdx3");
	self.mowers[3].idx1 = idx1;
	self.mowers[3].idx2 = idx2;
	self.mowers[3].idx3 = idx3;
	self.mowers[3].jointRotLimits = {};
	self.mowers[3].jointRotLimits[1] = Utils.degToRad(10);
	self.mowers[3].jointRotLimits[2] = Utils.degToRad(8);
	self.mowers[3].jointRotLimits[3] = Utils.degToRad(8);
	self.mowers[3].rotationNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.kroneBigM.right#rotateIdx"));
	local shutter1 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.kroneBigM.right#shutterIdx"));
	self.mowers[3].shutter1 = shutter1;	
	self.mowers[3].shutterIsOpen = false;
	self.mowers[3].shutterIsClosed = true;
	local roll1 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.kroneBigM.right#rollIdx"));
	self.mowers[3].roll1 = roll1;
	

	--### variables for swathPSnodes
	local file = getXMLString(xmlFile, "vehicle.kroneBigM.swathPS#file"); --particleSystems/swathFront.i3d" 
	self.swathPS = {};
	self.mowers[1].swathPSnode1 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.kroneBigM.swathPS#frontIdx1"));
	self.mowers[1].swathPSnode2 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.kroneBigM.swathPS#frontIdx2"));
	self.mowers[2].swathPSnode1 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.kroneBigM.swathPS#leftIdx1"));
	self.mowers[3].swathPSnode1 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.kroneBigM.swathPS#rightIdx1"));
	
	-- the PS
	self.mowersPS = {};
	for i=1,2 do
		self.mowersPS[i] = {};
		if i==1 then
			y = "normal";
		else 
			y = "swath";
		end;
		
		local node1 = Utils.indexToObject(self.components, getXMLString(xmlFile, string.format("vehicle.kroneBigM.ps.front.%s#frontIdx1", y)));
		local node2 = Utils.indexToObject(self.components, getXMLString(xmlFile, string.format("vehicle.kroneBigM.ps.front.%s#frontIdx2", y)));
		local node3 = Utils.indexToObject(self.components, getXMLString(xmlFile, string.format("vehicle.kroneBigM.ps.back.%s#leftIdx1", y)));
		local node4 = Utils.indexToObject(self.components, getXMLString(xmlFile, string.format("vehicle.kroneBigM.ps.back.%s#rightIdx1", y)));
		if node1 == nil or node2 == nil or node3 == nil or node4 == nil then 
			--print("Warning: No mower-particlesystems possible for fruitType: "..v.." "..y);
		else
			self.mowersPS[i].pss = {};
			
			local particleSystem1 = {};
			Utils.loadParticleSystem(xmlFile, particleSystem1, string.format("vehicle.kroneBigM.ps.front.%s", y), node1, false, nil, self.baseDirectory);
			self.mowersPS[i].pss[1] = particleSystem1;					

			local particleSystem2 = {};
			Utils.loadParticleSystem(xmlFile, particleSystem2, string.format("vehicle.kroneBigM.ps.front.%s", y), node2, false, nil, self.baseDirectory);
			self.mowersPS[i].pss[2] = particleSystem2;					

			local particleSystem3 = {};
			Utils.loadParticleSystem(xmlFile, particleSystem3, string.format("vehicle.kroneBigM.ps.back.%s", y), node3, false, nil, self.baseDirectory);
			self.mowersPS[i].pss[3] = particleSystem3;					

			local particleSystem4 = {};
			Utils.loadParticleSystem(xmlFile, particleSystem4, string.format("vehicle.kroneBigM.ps.back.%s", y), node4, false, nil, self.baseDirectory);
			self.mowersPS[i].pss[4] = particleSystem4;					
		end;
	end;
	
	--### load the hydraulic system
	local i=0;
	self.cylinders = {};
	while true do
		local girl = Utils.indexToObject( self.components, getXMLString(xmlFile, string.format("vehicle.kroneBigM.hydraulic.cylinder(%d)#girl",i)) );
		if girl == nil then
			break;
		end;
		local boy = Utils.indexToObject( self.components, getXMLString(xmlFile, string.format("vehicle.kroneBigM.hydraulic.cylinder(%d)#boy",i)) );
		local ref = Utils.indexToObject( self.components, getXMLString(xmlFile, string.format("vehicle.kroneBigM.hydraulic.cylinder(%d)#ref",i)) );
		local target = Utils.indexToObject( self.components, getXMLString(xmlFile, string.format("vehicle.kroneBigM.hydraulic.cylinder(%d)#target",i)) );
		
		local entry = {};
		entry.girl = girl;
		entry.boy = boy;
		entry.ref = ref;
		entry.target = target;
		local x1, y1, z1 = getWorldTranslation(boy);
		local x2, y2, z2 = getWorldTranslation(ref);
		entry.length2 = Utils.vector3Length(x1-x2, y1-y2, z1-z2);
		
		i=i+1;
		table.insert(self.cylinders, entry);
	end;
	
	
	--### the mowers areas + sound
    if self.isClient then
		self.mowerSoundEnabled = {};
		for i=1,3 do
			self.mowerSoundEnabled[i] = false;
		end;

		self.mowerSound = {};
		self.mowerSoundPitchOffset = {};
		self.mowerSoundVolume = {};
		
		local mowerSound = getXMLString(xmlFile, string.format("vehicle.mowerSound#file"));
		mowerSound = Utils.getFilename(mowerSound, self.baseDirectory); 
		self.mowerSound = createSample(string.format("mowerSound"));

		loadSample(self.mowerSound, mowerSound, false);
		self.mowerSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, string.format("vehicle.mowerSound#pitchOffset")), 1);
		self.mowerSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, string.format("vehicle.mowerSound#volume")), 1);
		
		self.mowerSoundEnabled = {};
		for i=1,3 do
			self.mowerSoundEnabled[i] = false;
		end;
		self.mowersActive = 0;
    end;

    self.groundReferenceThreshold = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.groundReferenceNode#threshold"), 0.4);
	
    local numCuttingAreas = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.cuttingAreas#count"), 0);
    self.cuttingAreas = {};
	self.groundReferenceNode = {};
    for i=1, numCuttingAreas do
        self.cuttingAreas[i] = {};
        local areanamei = string.format("vehicle.cuttingAreas.cuttingArea%d", i);
        self.cuttingAreas[i].start = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#startIndex"));
        self.cuttingAreas[i].width = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#widthIndex"));
        self.cuttingAreas[i].height = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#heightIndex"));
		self.groundReferenceNode[i] = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#groundRefIdx"));
	end;

	for i=1,4 do 
		if self.groundReferenceNode[i] == nil then
			self.groundReferenceNode[i] = self.components[1].node;
		end;
	end;

	for i=1,3 do 
		self.mowers[i].setTurnedOn = false;
	end;
    self.wasToFastForMowing = false;	
	
	
	--## windrower/swath areas + sound
	if self.isClient then
		self.windrowerSoundEnabled = {};
		for i=1,3 do
			self.windrowerSoundEnabled[i] = false;
		end;

		self.windrowerSound = {};
		self.windrowerSoundPitchOffset = {};
		self.windrowerSoundVolume = {};
		
		local windrowerSound = getXMLString(xmlFile, string.format("vehicle.windrowerSound#file"));
		windrowerSound = Utils.getFilename(windrowerSound, self.baseDirectory); 
		self.windrowerSound = createSample("windrowerSound");

		loadSample(self.windrowerSound, windrowerSound, false);
		self.windrowerSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, string.format("vehicle.windrowerSound#pitchOffset")), 1);
		self.windrowerSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, string.format("vehicle.windrowerSound#volume")), 1);
		
		self.windrowerSoundEnabled = {};
		for i=1,3 do
			self.windrowerSoundEnabled[i] = false;
		end;
		self.windrowersActive = 0;
    end;

    local numCuttingAreas = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.windrowerCuttingAreas#count"), 0);
    self.windrowerCuttingAreas = {};
	self.accumulatedCuttingAreaValues = {};
    for i=1, numCuttingAreas do
        self.windrowerCuttingAreas[i] = {};
        local areanamei = string.format("vehicle.windrowerCuttingAreas.cuttingArea%d", i);
        self.windrowerCuttingAreas[i].start = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#startIndex"));
        self.windrowerCuttingAreas[i].width = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#widthIndex"));
        self.windrowerCuttingAreas[i].height = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#heightIndex"));
		self.accumulatedCuttingAreaValues[i] = 0;
	end;
	
    local numWindrowerDropAreas = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.windrowerDropAreas#count"), 0);
    if numWindrowerDropAreas == 0 then
        print("Warning: No drop areas specified in '"..self.configFileName.."'");
    else
        if numWindrowerDropAreas ~= 1 and numWindrowerDropAreas ~= table.getn(self.windrowerCuttingAreas) then
            print("Warning: Number of cutting areas and drop areas should be equal in '"..self.configFileName.."'");
        end;
    end;
    self.windrowerDropAreas = {}
	self.accumulatedCuttingAreaValues = {};
    for i=1, numWindrowerDropAreas do
        self.windrowerDropAreas[i] = {};
        local areanamei = string.format("vehicle.windrowerDropAreas.windrowerDropArea%d", i);
        self.windrowerDropAreas[i].start = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#startIndex"));
        self.windrowerDropAreas[i].width = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#widthIndex"));
        self.windrowerDropAreas[i].height = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#heightIndex"));
		self.accumulatedCuttingAreaValues[i] = 0;
    end;	
	
	--#
	self.psStateSend = {false, false, false, false};
	self.doPSstateUpdate = false;
	
	--#
	self.letBackmowersFollow = false;
	self.liftTime = 0;
	
	
	--# finally a HUD 
	self.hud = {};
	local path = Utils.getFilename("textures/window_diffuse_Krone2.png", self.baseDirectory);
	self.hud.overlay = Overlay:new("hudOverlay", path, 0.75, 0.2128, 0.248, 0.25);
	self.hud.displayHUD = 1;

	
	--#cabine HUD = IC
	self.cabineHUD = {};
	for i=1,4 do
		local parts = {};
		local node1 = Utils.indexToObject(self.components, string.format("0>0|10|%d|0",3+i));
		parts.node1 = node1;
		local node2 = Utils.indexToObject(self.components, string.format("0>0|10|%d|1",3+i));
		parts.node2 = node2;
		local node3 = Utils.indexToObject(self.components, string.format("0>0|10|%d|2",3+i));
		parts.node3 = node3;
		local node4 = Utils.indexToObject(self.components, string.format("0>0|10|%d|3",3+i));
		parts.node4 = node4;
		table.insert(self.cabineHUD, parts);
	end;
	
	--#beacon
	self.beacons = {};
	for i=1,3 do
		local node = Utils.indexToObject(self.components, getXMLString(xmlFile, string.format("vehicle.beaconBody#index%d", i)));
		table.insert(self.beacons, node);
	end;
	


end;

function KroneBigM500:delete()
	stopSample(self.mowerSound);
	delete(self.mowerSound);
	stopSample(self.windrowerSound);
	delete(self.windrowerSound);	

	-- deleteParticleSystem ?!
end;

function KroneBigM500:readStream(streamId, connection)
	local activeMowerIdx = streamReadInt8(streamId);
	--self:setActiveMower(activeMowerIdx, true);
	for i=1, 4 do
		self.mowers[i].setUnfold = streamReadBool(streamId);
		self.mowers[i].setDown = streamReadBool(streamId);
		self.mowers[i].setTurnedOn = streamReadBool(streamId);
		self.mowers[i].setSwathOn = streamReadBool(streamId);
		self.mowers[i].isTransport = streamReadBool(streamId);
		self.mowers[i].isWorking = streamReadBool(streamId);
		self.mowers[i].isUp = streamReadBool(streamId);
		self.mowers[i].isDown = streamReadBool(streamId);
		--self:setActiveMower(i, true);
		--self:setMowerOptions(	self.mowers[i].setUnfold, self.mowers[i].setDown, 
		--						self.mowers[i].setTurnedOn, self.mowers[i].setSwathOn, true);
					
	end;
	self:setActiveMower(activeMowerIdx, true);
	
	for i=2,3 do
		local rx = streamReadFloat32(streamId);
		local ry = streamReadFloat32(streamId);
		local rz = streamReadFloat32(streamId);
		setRotation(self.mowers[i].rotationNode, rx, ry, rz);
	end;
	
	local r1 = streamReadFloat32(streamId);
	local r2 = streamReadFloat32(streamId);
	setJointRotationLimit(self.componentJoints[self.mowers[1].idx1].jointIndex, 2, true, -r1, r1);
	setJointRotationLimit(self.componentJoints[self.mowers[1].idx2].jointIndex, 2, true, -r2, r2);	
	
	for i=2,3 do
		local r1 = streamReadFloat32(streamId);
		local r2 = streamReadFloat32(streamId);
		local r3 = streamReadFloat32(streamId);
		setJointRotationLimit(self.componentJoints[self.mowers[i].idx1].jointIndex, 2, true, -r1, r1);
		setJointRotationLimit(self.componentJoints[self.mowers[i].idx2].jointIndex, 2, true, -r1, r1);		
		setJointRotationLimit(self.componentJoints[self.mowers[i].idx3].jointIndex, 0, true, -r2, r2);	
		setJointRotationLimit(self.componentJoints[self.mowers[i].idx3].jointIndex, 2, true, -r3, r3);			
	end	
	
	for i=1,10 do
		local rx = streamReadFloat32(streamId);
		local ry = streamReadFloat32(streamId);
		local rz = streamReadFloat32(streamId);
		setRotation(self.componentJoints[i].jointNode, rx, ry, rz);
		setJointFrame(self.componentJoints[i].jointIndex, 0, self.componentJoints[i].jointNode);	
	end;	

end;


function KroneBigM500:writeStream(streamId, connection)
	streamWriteInt8(streamId, self.activeMowerIdx);
	for i=1, 4 do
		streamWriteBool(streamId, self.mowers[i].setUnfold);
		streamWriteBool(streamId, self.mowers[i].setDown);
		streamWriteBool(streamId, self.mowers[i].setTurnedOn);
		streamWriteBool(streamId, self.mowers[i].setSwathOn);
		streamWriteBool(streamId, self.mowers[i].isTransport);
		streamWriteBool(streamId, self.mowers[i].isWorking);
		streamWriteBool(streamId, self.mowers[i].isUp);
		streamWriteBool(streamId, self.mowers[i].isDown);
	end;
	
	for i=2,3 do
		local rx, ry, rz = getRotation(self.mowers[i].rotationNode);
		streamWriteFloat32(streamId, rx);
		streamWriteFloat32(streamId, ry);
		streamWriteFloat32(streamId, rz);
	end;
	
	streamWriteFloat32(streamId, self.mowers[1].jointRotLimit);
	streamWriteFloat32(streamId, self.mowers[1].jointRotLimit2);
	
	for i=2,3 do
		streamWriteFloat32(streamId, self.mowers[i].jointRotLimits[1]);
		streamWriteFloat32(streamId, self.mowers[i].jointRotLimits[2]);
		streamWriteFloat32(streamId, self.mowers[i].jointRotLimits[3]);
	end
	
	
	for i=1,10 do
		local rx, ry, rz = getRotation(self.componentJoints[i].jointNode);
		streamWriteFloat32(streamId, rx);
		streamWriteFloat32(streamId, ry);
		streamWriteFloat32(streamId, rz);
	end;	
end;

function KroneBigM500:readStreamUpdate(streamId, connection)
end;

function KroneBigM500:writeStreamUpdate(streamId, connection)
end;

function KroneBigM500:mouseEvent(posX, posY, isDown, isUp, button)
end;

function KroneBigM500:keyEvent(unicode, sym, modifier, isDown)
end;

function KroneBigM500:update(dt)

	if self:getIsActive() and self:getIsActiveForInput() then
		local idx = self.activeMowerIdx;
		if InputBinding.hasEvent(InputBinding.BIGM500_SELECT) then
			idx = idx + 1;
			if idx > table.getn(self.mowerTypes) then
				idx = 1;
			end;
			self:setActiveMower(idx);
		end;
		if InputBinding.hasEvent(InputBinding.BIGM500_FOLD) then
			self:setMowerOptions(not self.mowers[idx].setUnfold, self.mowers[idx].setDown, self.mowers[idx].setTurnedOn, self.mowers[idx].setSwathOn);
		end;
		if InputBinding.hasEvent(InputBinding.BIGM500_UPDOWN) then
			--if idx == table.getn(self.mowerTypes) then
			--	idx = 1;
			--	self.letBackmowersFollow = true;
			--	self.liftTime = 0;
			--end;
			--self:setActiveMower(idx);
			self:setMowerOptions(self.mowers[idx].setUnfold, not self.mowers[idx].setDown, self.mowers[idx].setTurnedOn, self.mowers[idx].setSwathOn);
		end;
		if InputBinding.hasEvent(InputBinding.BIGM500_ONOFF) then
			self:setMowerOptions(self.mowers[idx].setUnfold, self.mowers[idx].setDown, not self.mowers[idx].setTurnedOn, self.mowers[idx].setSwathOn);
		end;
		if InputBinding.hasEvent(InputBinding.BIGM500_SWATH) then
			self:setMowerOptions(self.mowers[idx].setUnfold, self.mowers[idx].setDown, self.mowers[idx].setTurnedOn, not self.mowers[idx].setSwathOn);
		end;
		if InputBinding.hasEvent(InputBinding.BIGM500_HUD) then
			self.hud.displayHUD = self.hud.displayHUD + 1;
			if self.hud.displayHUD > 1 then
				self.hud.displayHUD = 0;
			end;
		end;
		
		local tmp = self.activeMowerIdx;
		if InputBinding.hasEvent(InputBinding.BIGM500_FRONT) then
			self:setActiveMower(1);
			self:setMowerOptions(self.mowers[1].setUnfold, not self.mowers[1].setDown, self.mowers[1].setTurnedOn, self.mowers[1].setSwathOn);
			self:setActiveMower(tmp);
		end;
		if InputBinding.hasEvent(InputBinding.BIGM500_LEFT) then
			self:setActiveMower(2);
			self:setMowerOptions(self.mowers[2].setUnfold, not self.mowers[2].setDown, self.mowers[2].setTurnedOn, self.mowers[2].setSwathOn);
			self:setActiveMower(tmp);
		end;
		if InputBinding.hasEvent(InputBinding.BIGM500_RIGHT) then
			self:setActiveMower(3);
			self:setMowerOptions(self.mowers[3].setUnfold, not self.mowers[3].setDown, self.mowers[3].setTurnedOn, self.mowers[3].setSwathOn);
			self:setActiveMower(tmp);
		end;		
		
		
		if self.letBackmowersFollow == true then
			self.liftTime = self.liftTime + dt;
			if self.liftTime > 1000 then
				idx = table.getn(self.mowerTypes);
				self:setActiveMower(idx);
				self:setMowerOptions(self.mowers[idx].setUnfold, not self.mowers[idx].setDown, self.mowers[idx].setTurnedOn, self.mowers[idx].setSwathOn);
				self.liftTime = 0;
				self.letBackmowersFollow = false;				
			end;
		end;
	end;
end;


function KroneBigM500:updateTick(dt)

	--####
	if self.isServer then
		for k,wheel in pairs(self.wheels) do
			if k>4 then 
				setWheelShapeProps(wheel.node, wheel.wheelShape, 0, 0, 0);
			end;
		end;
	end;
 
	--### de-/activate the mower(s)
	local lastSpeed = self.lastSpeed*3600;
	local toFast = self:doCheckSpeedLimit() and lastSpeed > 24;  --self.lastSpeed*3600 > 24;
	local check = false;
	for i=1,3 do
		if self.mowers[i].setTurnedOn and self.mowers[i].isDown then
			check = true;
		end;
	end;
	if check == true then
		self.wasToFastForMowing = toFast;
	else
		self.wasToFastForMowing = false;
	end;

	if self.isServer then
	
		local cuttingAreasSend = {};
		local windrowerCuttingAreasSend = {};
		local windrowerDropAreasSend = {};	
		local validCutters = {false, false, false, false};
	
		if self.wasToFastForMowing == false and self:getIsActive() and self.movingDirection >= 0 then
				
			for i=1,4 do
			
				local j=i;
				if i > 1 then
					j = i-1;
				end;
				
				if self.mowers[j].isWorking == false and self.mowers[j].setTurnedOn then
					self.mowers[j].setTurnedOn = false;
				end;
				
				if self.mowers[j].setTurnedOn then
				
					local rx,ry,rz = getWorldTranslation(self.groundReferenceNode[i]);
					local terrainHeight = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, rx, 0, rz);
					
					if terrainHeight+self.groundReferenceThreshold >= ry then
				
						local cuttingArea = self.cuttingAreas[i];
						--if self:getIsAreaActive(cuttingArea) then
							local x,y,z = getWorldTranslation(cuttingArea.start);
							local x1,y1,z1 = getWorldTranslation(cuttingArea.width);
							local x2,y2,z2 = getWorldTranslation(cuttingArea.height);
							table.insert(cuttingAreasSend, {x,z,x1,z1,x2,z2});
							local area = Utils.getFruitArea(FruitUtil.FRUITTYPE_GRASS, x, z, x1, z1, x2, z2);
							if area > 0 then
								validCutters[i] = true;
							end;
						--end;

						if self.mowers[j].setSwathOn == true then
							local cuttingArea = self.windrowerCuttingAreas[i];
							--if self:getIsAreaActive(cuttingArea) then
								local x,y,z = getWorldTranslation(cuttingArea.start);
								local x1,y1,z1 = getWorldTranslation(cuttingArea.width);
								local x2,y2,z2 = getWorldTranslation(cuttingArea.height);
								local area = Utils.getFruitCutLongArea(FruitUtil.FRUITTYPE_GRASS, x, z, x1, z1, x2, z2);
								local area2 = Utils.getFruitWindrowArea(FruitUtil.FRUITTYPE_GRASS, x, z, x1, z1, x2, z2);
								if area > 0 or area2 > 0 then
									validCutters[i] = true;
									table.insert(windrowerCuttingAreasSend, {x,z,x1,z1,x2,z2});			
									local dropArea = self.windrowerDropAreas[i];									
									local x,y,z = getWorldTranslation(dropArea.start);
									local x1,y1,z1 = getWorldTranslation(dropArea.width);
									local x2,y2,z2 = getWorldTranslation(dropArea.height);
									table.insert(windrowerDropAreasSend, {x,z,x1,z1,x2,z2});
								end;
							--end;
						end;

					end;
				end;
				
			end;
			
			if (table.getn(cuttingAreasSend) > 0) then
				MowerAreaEvent.runLocally(cuttingAreasSend);
				g_server:broadcastEvent(MowerAreaEvent:new(cuttingAreasSend));
			end;
			
			if (table.getn(windrowerCuttingAreasSend) > 0) then
				local fruitType = FruitUtil.FRUITTYPE_GRASS;
				local cuttingAreasSend, dropAreasSend, fruitType, bitType = WindrowAreaEvent.runLocally(windrowerCuttingAreasSend, windrowerDropAreasSend, self.accumulatedCuttingAreaValues);
				if table.getn(cuttingAreasSend) > 0 then
					g_server:broadcastEvent(WindrowAreaEvent:new(cuttingAreasSend, dropAreasSend, fruitType, bitType));
				end;
			end;
		end;
		--do MP-updates for the PSs ..
		self:setCuttersState(validCutters);
	end;	
				
	if self.isClient then	
		local cntp = 0;
		local cntn = 0;

		for i=1,3 do
			if not self.mowerSoundEnabled[i] and self.mowers[i].setTurnedOn and not self.mowers[i].setSwathOn and self:getIsActiveForSound() then
				self.mowerSoundEnabled[i] = true;
				cntp = cntp + 1;
			end;
			if (not self.mowers[i].setTurnedOn or self.mowers[i].setSwathOn) and self.mowerSoundEnabled[i] then
				self.mowerSoundEnabled[i] = false;
				cntn = cntn + 1;
			end;
		end;
		local mowersActive = self.mowersActive + cntp - cntn;
		if mowersActive ~= self.mowersActive then
			stopSample(self.mowerSound);
			if mowersActive > 0 then
				setSamplePitch(self.mowerSound, self.mowerSoundPitchOffset);
				playSample(self.mowerSound, 0, self.mowerSoundVolume+0.12*mowersActive, 0);
			end;
			self.mowersActive = mowersActive;
		end;

		cntp = 0;
		cntn = 0;
		for i=1,3 do
			if not self.windrowerSoundEnabled[i] and self.mowers[i].setSwathOn and self.mowers[i].setTurnedOn and self:getIsActiveForSound() then
				self.windrowerSoundEnabled[i] = true;
				cntp = cntp + 1;
			end;
			if (not self.mowers[i].setSwathOn or not self.mowers[i].setTurnedOn) and self.windrowerSoundEnabled[i] then
				self.windrowerSoundEnabled[i] = false;
				cntn = cntn + 1;
			end;				
		end;
		local windrowersActive = self.windrowersActive + cntp - cntn;
		if windrowersActive ~= self.windrowersActive then
			stopSample(self.windrowerSound);
			if windrowersActive > 0 then 
				setSamplePitch(self.windrowerSound, self.windrowerSoundPitchOffset);
				playSample(self.windrowerSound, 0, self.windrowerSoundVolume+0.11*windrowersActive, 0);
			end;
			self.windrowersActive = windrowersActive;
		end;
		
	end; 
	
		
	--# set PS on/off
	for i=1, 4 do
		local j=i;
		if i > 1 then 	
			j = i-1;
		end;
		if self.mowers[j].isWorking and self.mowers[j].setTurnedOn == true and 
			self.psStateSend[i] == true then
			if self.mowers[j].setSwathOn == false then
				Utils.setEmittingState(self.mowersPS[1].pss[i], true);	
				Utils.setEmittingState(self.mowersPS[2].pss[i], false);	
			else
				Utils.setEmittingState(self.mowersPS[1].pss[i], false);	
				Utils.setEmittingState(self.mowersPS[2].pss[i], true);	
			end;
		else 
			Utils.setEmittingState(self.mowersPS[1].pss[i], false);	
			Utils.setEmittingState(self.mowersPS[2].pss[i], false);	
		end;
	end;
	
	
	--#setSwathOn/Off
	for i=1, 3 do
			
		if self.mowers[i].isWorking == false then
			self.mowers[i].setSwathOn = true;
		end;
		
		local open = (not self.mowers[i].setSwathOn) and self.mowers[i].isWorking;
		local rx, ry, rz = getRotation(self.mowers[i].shutter1);
		local maxValue; -- = Utils.degToRad(35);
		if i > 1 then
			maxValue = Utils.degToRad(35);
		else
			maxValue = Utils.degToRad(15);
		end;
		local rotVal = Utils.getMovedLimitedValues({rx}, {Utils.degToRad(0)}, {maxValue}, 1, 2000, dt, open);
		setRotation(self.mowers[i].shutter1, rotVal[1], ry, rz);
		if i == 1 then
			setRotation(self.mowers[i].shutter2, rotVal[1], ry, rz);
		end;
		
		rx, ry, rz = getRotation(self.mowers[i].roll1);
		setRotation(self.mowers[i].roll1, rx-0.02, ry, rz);
		if i == 1 then
			setRotation(self.mowers[i].roll2, rx-0.02, ry, rz);
		end;
	end;
	

	--if self.isServer then
	
		--### first the front mower ------

		
		--#set up/down or auto if working
		
	if self.mowers[1].isWorking == true then
		if self.mowers[1].setDown == true and self.mowers[1].isDown == false then
			self.animationParts[1].clipSpeed = 2.0;
			if self.isServer then
				self:setAnimationTime(1, 0, false); -- for auto movement 
				--self:setAnimationTime(1, self.animationParts[1].animDuration/2, false); -- for no movement 
			end;
			self.mowers[1].isUp = false;
			local rotLimit = Utils.getMovedLimitedValues({self.mowers[1].jointRotLimit2}, {Utils.degToRad(20)}, {0}, 1, 2000, dt, false);
			if self.isServer then
				setJointRotationLimit(self.componentJoints[1].jointIndex, 0, true, -rotLimit[1], rotLimit[1]);
				setJointRotationLimit(self.componentJoints[2].jointIndex, 0, true, -rotLimit[1], rotLimit[1]);	
			end;
			self.mowers[1].jointRotLimit2 = rotLimit[1];
			local xj, yj, zj = getWorldTranslation(self.componentJoints[1].jointNode);			
			local hj = yj - getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, xj, yj, zj);
			local delta = 0.1;
			if self.animationParts[1].currentPosition <= 0 or hj < self.frontMower.height + delta then
				if self.isServer then
					self:setAnimationTime(1, self.animationParts[1].currentPosition, false);
					rotLimit[1] = Utils.degToRad(20);
					setJointRotationLimit(self.componentJoints[1].jointIndex, 0, true, -rotLimit[1], rotLimit[1]);
					setJointRotationLimit(self.componentJoints[2].jointIndex, 0, true, -rotLimit[1], rotLimit[1]);					
				end;
				self.mowers[1].isDown = true;
				self.frontMower.moveAuto = true;	
				--print("self.mowers[1].isDown = true;");
					-- !!!
				--end;
			end;			
		elseif self.mowers[1].setDown == false and self.mowers[1].isUp == false then
			self.animationParts[1].clipSpeed = 2.0;
			if self.isServer then
				self:setAnimationTime(1, self.animationParts[1].animDuration, false);
			end;
			self.frontMower.moveAuto = false;
			self.mowers[1].isDown = false;
			local rotLimit = Utils.getMovedLimitedValues({self.mowers[1].jointRotLimit2}, {Utils.degToRad(20)}, {0}, 1, 2000, dt, true);
			if self.isServer then
				setJointRotationLimit(self.componentJoints[1].jointIndex, 0, true, -rotLimit[1], rotLimit[1]);
				setJointRotationLimit(self.componentJoints[2].jointIndex, 0, true, -rotLimit[1], rotLimit[1]);	
			end;
			self.mowers[1].jointRotLimit2 = rotLimit[1];
			if self.animationParts[1].currentPosition >= self.animationParts[1].animDuration then
				if self.isServer then
					self:setAnimationTime(1, self.animationParts[1].animDuration, false);
				end;
				if self.mowers[1].jointRotLimit2 == Utils.degToRad(0) then
					self.mowers[1].isUp = true;
					--print("self.mowers[1].isUp = true;");
				end;
			end;					
		end;
	end;
	
	if self.mowers[1].isDown == true and self.frontMower.moveAuto == true and self.mowers[1].isWorking == true then
		self.animationParts[1].clipSpeed = 1.0;
		if self.isServer then

			local x, y, z = getWorldTranslation(self.frontMower.node1);
			local h1 = y - getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, x, y, z);
			
			local x2, y2, z2 = getWorldTranslation(self.frontMower.node2);
			local h2 = y2 - getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, x2, y2, z2);

			local xj, yj, zj = getWorldTranslation(self.componentJoints[1].jointNode);
			local hj = yj - getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, xj, yj, zj);

			local delta = 0.1;
			--local stepsize = math.max(math.min(dt, 60), 20);
			local stepsize = dt;
			
			--print("-->auto: ");
			 if hj < self.frontMower.height + 2*delta or h2 < self.frontMower.height + 2*delta or h1 < self.frontMower.height + 2*delta then
				--print("->if");
				if self.animationParts[1].currentPosition < self.animationParts[1].animDuration - stepsize then
					self:setAnimationTime(1, math.min(self.animationParts[1].animDuration, self.animationParts[1].currentPosition + stepsize), false); 
				end;
			--else
				--if (y2 > y + 3*delta) then --and hj < self.frontMower.height + 3*delta then 
					--print("->else if");
					--self:setAnimationTime(1, self.animationParts[1].currentPosition + dt, false); 
				--elseif (hj > self.frontMower.height + 3*delta or (y2 < y - 3*delta) ) or
					--	(hj > self.frontMower.height + 2*delta and (y2 < y - 1*delta) )	then
					--print("->else elseif");
					--self:setAnimationTime(1, self.animationParts[1].currentPosition - dt, false); 
				--end 
			elseif (hj > self.frontMower.height + 4*delta) then
				if self.animationParts[1].currentPosition > stepsize then
					self:setAnimationTime(1, math.max(0, self.animationParts[1].currentPosition - stepsize), false); 
				end;
			end; 
		end;
	end; 

	--always?
	--if self.mowers[1].isWorking == false then
	if (self.mowers[1].isWorking == true and self.mowers[1].setDown == false) or 
		(self.mowers[1].isWorking == true and self.mowers[1].setDown == true and self.mowers[1].isDown == false) then
		local rx, ry, rz = getRotation(self.frontMower.hydraulicNode);
		setRotation(self.componentJoints[1].jointNode, -rx, 0, 0);
		setRotation(self.componentJoints[2].jointNode, -rx, 0, 0);
		if self.isServer then
			setJointFrame(self.componentJoints[1].jointIndex, 0, self.componentJoints[1].jointNode);
			setJointFrame(self.componentJoints[2].jointIndex, 0, self.componentJoints[2].jointNode);			
		end;
	end;
		
	--#setTransport/Working
	if self.mowers[1].isUp == true then
	
		if self.mowers[1].setUnfold == false and self.mowers[1].isTransport == false then
			self.mowers[1].isWorking = false;
			self.mowers[1].setTurnedOn = false;
			self.animationParts[1].clipSpeed = 0.25;
	
			local rx1, ry1, rz1 = getRotation(self.componentJoints[self.mowers[1].idx1].jointNode);
			local rx2, ry2, rz2 = getRotation(self.componentJoints[self.mowers[1].idx2].jointNode);
			local minAngle1 = -Utils.degToRad(75);
			local maxAngle1 = 0;
			local minAngle2 = 0;
			local maxAngle2 = Utils.degToRad(75);
			
			local new1 = Utils.getMovedLimitedValues({rz1}, {maxAngle1}, {minAngle1}, 1, 10000, dt, true);
			local new2 = Utils.getMovedLimitedValues({rz2}, {maxAngle2}, {minAngle2}, 1, 10000, dt, false);
			
			setRotation(self.componentJoints[self.mowers[1].idx1].jointNode, rx1, ry1, new1[1]);
			setRotation(self.componentJoints[self.mowers[1].idx2].jointNode, rx2, ry2, new2[1]);
			
			if self.isServer then
				setJointFrame(self.componentJoints[self.mowers[1].idx1].jointIndex, 0, self.componentJoints[self.mowers[1].idx1].jointNode);
				setJointFrame(self.componentJoints[self.mowers[1].idx2].jointIndex, 0, self.componentJoints[self.mowers[1].idx2].jointNode);	
			end

			local rotLimit = Utils.getMovedLimitedValues({self.mowers[1].jointRotLimit}, {Utils.degToRad(8)}, {0}, 1, 10000, dt, true);
			if self.isServer then
				setJointRotationLimit(self.componentJoints[self.mowers[1].idx1].jointIndex, 2, true, -rotLimit[1], rotLimit[1]);
				setJointRotationLimit(self.componentJoints[self.mowers[1].idx2].jointIndex, 2, true, -rotLimit[1], rotLimit[1]);	
			end;
			self.mowers[1].jointRotLimit = rotLimit[1];					

			if self.mowers[1].jointRotLimit == 0 then 
				if new1[1] == minAngle1 and new2[1] == maxAngle2 then
					
					if self.isServer then
						self:setAnimationTime(1, self.animationParts[1].animDuration/4, false); 
					end;
					
					if self.animationParts[1].currentPosition <= 3*self.animationParts[1].animDuration/4 then
						local rx, ry, rz = getRotation(self.componentJoints[1].jointNode);
						local new = Utils.getMovedLimitedValues({rx}, {Utils.degToRad(28)}, {Utils.degToRad(-12)}, 1, 7000, dt, true);
						setRotation(self.componentJoints[1].jointNode, new[1], ry, rz);
						setRotation(self.componentJoints[2].jointNode, new[1], ry, rz);
						if self.isServer then
							setJointFrame(self.componentJoints[1].jointIndex, 0, self.componentJoints[1].jointNode);
							setJointFrame(self.componentJoints[2].jointIndex, 0, self.componentJoints[2].jointNode);
						end;
						--local rx, ry, rz = getRotation(self.componentJoints[1].jointNode);
						--if new[1] <= Utils.degToRad(-12) + 0.001 and self.animationParts[1].currentPosition <= (self.animationParts[1].animDuration/4 + self.animationParts[1].offSet) then
						if new[1] == Utils.degToRad(-12) and self.animationParts[1].currentPosition <= (self.animationParts[1].animDuration/4 + self.animationParts[1].offSet) then
							self.mowers[1].isTransport = true;
							--print("self.mowers[1].isTransport = true;");
						end;
					end;
				end;
			end;
			
		elseif self.mowers[1].setUnfold == true and self.mowers[1].isWorking == false then
			self.mowers[1].isTransport = false;
			self.animationParts[1].clipSpeed = 0.25;
	
			local rx, ry, rz = getRotation(self.componentJoints[1].jointNode);
			local new = Utils.getMovedLimitedValues({rx}, {Utils.degToRad(28)}, {Utils.degToRad(-12)}, 1, 7000, dt, false);
			setRotation(self.componentJoints[1].jointNode, new[1], ry, rz);
			setRotation(self.componentJoints[2].jointNode, new[1], ry, rz);
			if self.isServer then
				setJointFrame(self.componentJoints[1].jointIndex, 0, self.componentJoints[1].jointNode);
				setJointFrame(self.componentJoints[2].jointIndex, 0, self.componentJoints[2].jointNode);
			end;

			--print("rx="..rx.." min/max="..Utils.degToRad(-12).."/"..Utils.degToRad(28).." t:"..self.animationParts[1].currentPosition);
			--local rx, ry, rz = getRotation(self.componentJoints[1].jointNode);
			if new[1] > Utils.degToRad(5) then
				if self.isServer then
					self:setAnimationTime(1, self.animationParts[1].animDuration, false); 
				end;
							
				if (new[1] >= Utils.degToRad(28) - 0.001) and (self.animationParts[1].currentPosition >= self.animationParts[1].animDuration - self.animationParts[1].offSet) then
				
					local rx1, ry1, rz1 = getRotation(self.componentJoints[self.mowers[1].idx1].jointNode);
					local rx2, ry2, rz2 = getRotation(self.componentJoints[self.mowers[1].idx2].jointNode);
					local minAngle1 = -Utils.degToRad(75);
					local maxAngle1 = 0;
					local minAngle2 = 0;
					local maxAngle2 = Utils.degToRad(75);
					
					local new1 = Utils.getMovedLimitedValues({rz1}, {maxAngle1}, {minAngle1}, 1, 10000, dt, false);
					local new2 = Utils.getMovedLimitedValues({rz2}, {maxAngle2}, {minAngle2}, 1, 10000, dt, true);
					
					setRotation(self.componentJoints[self.mowers[1].idx1].jointNode, rx1, ry1, new1[1]);
					setRotation(self.componentJoints[self.mowers[1].idx2].jointNode, rx2, ry2, new2[1]);
					if self.isServer then
						setJointFrame(self.componentJoints[self.mowers[1].idx1].jointIndex, 0, self.componentJoints[self.mowers[1].idx1].jointNode);
						setJointFrame(self.componentJoints[self.mowers[1].idx2].jointIndex, 0, self.componentJoints[self.mowers[1].idx2].jointNode);					
					end;
					
					local rotLimit = Utils.getMovedLimitedValues({self.mowers[1].jointRotLimit}, {Utils.degToRad(8)}, {0}, 1, 10000, dt, false);
					if self.isServer then
						setJointRotationLimit(self.componentJoints[self.mowers[1].idx1].jointIndex, 2, true, -rotLimit[1], rotLimit[1]);
						setJointRotationLimit(self.componentJoints[self.mowers[1].idx2].jointIndex, 2, true, -rotLimit[1], rotLimit[1]);	
					end;
					self.mowers[1].jointRotLimit = rotLimit[1];		
										
					if self.mowers[1].jointRotLimit == Utils.degToRad(8) then -- 0.001 then 
						if new1[1] == maxAngle1 and new2[1] == minAngle2 then
							self.mowers[1].isWorking = true;
							--print("self.mowers[1].isWorking = true;");
						end;
					end;
				end;
			end;
			
		end;
		
	end;
		

		
		
	--####---------------------------------------------------------------------
	--#### left and right mower -----------------------------------------------
	
	local maxAngle = Utils.degToRad(15);
	local minAngle = Utils.degToRad(0);
		
	local limitsMin = {};
	local limitsMax = {};
	limitsMin[1] = 0;	limitsMax[1] = Utils.degToRad(15);
	limitsMin[2] = 0;	limitsMax[2] = Utils.degToRad(8);
	limitsMin[3] = 0;	limitsMax[3] = Utils.degToRad(8);

	for i=2,3 do
		
		if i==3 then
			maxAngle = Utils.degToRad(-15);
		end;
		
		local rx1, ry1, rz1 = getRotation(self.componentJoints[self.mowers[i].idx1].jointNode);
		local rx2, ry2, rz2 = getRotation(self.componentJoints[self.mowers[i].idx2].jointNode);
	
	
		if self.mowers[i].setDown == true and self.mowers[i].isDown == false and self.mowers[i].isWorking == true then
			self.mowers[i].isUp = false;
			local new1 = Utils.getMovedLimitedValues({rz1}, {maxAngle}, {minAngle}, 1, 3000, dt, true);
			local new2 = Utils.getMovedLimitedValues({rz2}, {maxAngle}, {minAngle}, 1, 3000, dt, true);
			
			setRotation(self.componentJoints[self.mowers[i].idx2].jointNode, rx2, ry2, new2[1]);
			setRotation(self.componentJoints[self.mowers[i].idx1].jointNode, rx1, ry1, new1[1]);

			if self.isServer then
				setJointFrame(self.componentJoints[self.mowers[i].idx1].jointIndex, 0, self.componentJoints[self.mowers[i].idx1].jointNode);
				setJointFrame(self.componentJoints[self.mowers[i].idx2].jointIndex, 0, self.componentJoints[self.mowers[i].idx2].jointNode);
			end;
		
			local rotLimits = Utils.getMovedLimitedValues(self.mowers[i].jointRotLimits, limitsMin, limitsMax, 3, 3000, dt, true);
			self.mowers[i].jointRotLimits = rotLimits;				
			
			if self.isServer then
				setJointRotationLimit(self.componentJoints[self.mowers[i].idx1].jointIndex, 2, true, -rotLimits[1], rotLimits[1]);
				setJointRotationLimit(self.componentJoints[self.mowers[i].idx2].jointIndex, 2, true, -rotLimits[1], rotLimits[1]);		
			
				setJointRotationLimit(self.componentJoints[self.mowers[i].idx3].jointIndex, 0, true, -rotLimits[2], rotLimits[2]);	
				setJointRotationLimit(self.componentJoints[self.mowers[i].idx3].jointIndex, 2, true, -rotLimits[3], rotLimits[3]);	
			end;
			
			if math.abs(new1[1] - minAngle) < 0.001 and math.abs(new2[1] - minAngle) < 0.001 then
				if self.mowers[i].jointRotLimits[1] > limitsMax[1] - 0.001 and
					self.mowers[i].jointRotLimits[2] > limitsMax[2] - 0.001 and
					self.mowers[i].jointRotLimits[3] > limitsMax[3] - 0.001 then
					self.mowers[i].isDown = true;
					--print("self.mowers["..i.."].isDown = true;");
				end;
			end;
			

		elseif self.mowers[i].setDown == false and self.mowers[i].isUp == false and self.mowers[i].isWorking == true then
			self.mowers[i].isDown = false;
			local new1 = Utils.getMovedLimitedValues({rz1}, {maxAngle}, {minAngle}, 1, 2200, dt, false);
			local new2 = Utils.getMovedLimitedValues({rz2}, {maxAngle}, {minAngle}, 1, 2200, dt, false);

			setRotation(self.componentJoints[self.mowers[i].idx1].jointNode, rx1, ry1, new1[1]);
			setRotation(self.componentJoints[self.mowers[i].idx2].jointNode, rx2, ry2, new2[1]);

			if self.isServer then
				setJointFrame(self.componentJoints[self.mowers[i].idx1].jointIndex, 0, self.componentJoints[self.mowers[i].idx1].jointNode);
				setJointFrame(self.componentJoints[self.mowers[i].idx2].jointIndex, 0, self.componentJoints[self.mowers[i].idx2].jointNode);
			end;
		
			local limitsMin = {};
			local limitsMax = {};
			limitsMin[1] = 0;	limitsMax[1] = Utils.degToRad(15);
			limitsMin[2] = 0;	limitsMax[2] = Utils.degToRad(8);
			limitsMin[3] = 0;	limitsMax[3] = Utils.degToRad(8);
			local rotLimits = Utils.getMovedLimitedValues(self.mowers[i].jointRotLimits, limitsMin, limitsMax, 3, 2200, dt, false);
			self.mowers[i].jointRotLimits = rotLimits;				
			
			if self.isServer then
				setJointRotationLimit(self.componentJoints[self.mowers[i].idx1].jointIndex, 2, true, -rotLimits[1], rotLimits[1]);
				setJointRotationLimit(self.componentJoints[self.mowers[i].idx2].jointIndex, 2, true, -rotLimits[1], rotLimits[1]);		
			
				setJointRotationLimit(self.componentJoints[self.mowers[i].idx3].jointIndex, 0, true, -rotLimits[2], rotLimits[2]);	
				setJointRotationLimit(self.componentJoints[self.mowers[i].idx3].jointIndex, 2, true, -rotLimits[3], rotLimits[3]);	
			end;

			if math.abs(new1[1] - maxAngle) < 0.001 and math.abs(new2[1] - maxAngle) < 0.001 then
				if self.mowers[i].jointRotLimits[1] < limitsMin[1] + 0.001 and
					self.mowers[i].jointRotLimits[2] < limitsMin[2] + 0.001 and
					self.mowers[i].jointRotLimits[3] < limitsMin[3] + 0.001 then
					self.mowers[i].isUp = true;
					--print("self.mowers["..i.."].isUp = true;");
				end;
			end;		
		end;
		
		--# left transport/working
		if self.mowers[i].isUp == true then
		
			local maxVal1 = Utils.degToRad(87.5);
			local maxVal2 = Utils.degToRad(-90);
			if i == 3 then
				maxVal1 = -maxVal1;
				maxVal2 = -maxVal2;
			end;
		
			if self.mowers[i].setUnfold == false and self.mowers[i].isTransport == false then
				self.mowers[i].isWorking = false;
				self.mowers[i].setTurnedOn = false;
				
				local rx1, ry1, rz1 = getRotation(self.componentJoints[self.mowers[i].idx1].jointNode);
				local newValues1 = {};
				if i == 2 then
					newValues1 = Utils.getMovedLimitedValues({rz1}, {maxAngle}, {maxVal1}, 1, 20000, dt, true);
				else
					newValues1 = Utils.getMovedLimitedValues({rz1}, {maxVal1}, {maxAngle}, 1, 20000, dt, false);
				end;

				setRotation(self.componentJoints[self.mowers[i].idx1].jointNode, rx1, ry1, newValues1[1]);
				setRotation(self.componentJoints[self.mowers[i].idx2].jointNode, rx1, ry1, newValues1[1]);
				if self.isServer then
					setJointFrame(self.componentJoints[self.mowers[i].idx1].jointIndex, 0, self.componentJoints[self.mowers[i].idx1].jointNode);
					setJointFrame(self.componentJoints[self.mowers[i].idx2].jointIndex, 0, self.componentJoints[self.mowers[i].idx2].jointNode);
				end;

				local rx2, ry2, rz2 = getRotation(self.mowers[i].rotationNode);
				local newValues2 = {};
				if i == 2 then
					newValues2 = Utils.getMovedLimitedValues({ry2}, {maxVal2}, {0}, 1, 20000, dt, false);
				else
					newValues2 = Utils.getMovedLimitedValues({ry2}, {0}, {maxVal2}, 1, 20000, dt, true);
				end;
				
				setRotation(self.mowers[i].rotationNode, rx2, newValues2[1], rz2);
				if self.isServer then
					setJointFrame(self.componentJoints[self.mowers[i].idx3].jointIndex, 0, self.componentJoints[self.mowers[i].idx3].jointNode);
				end;
				
				if newValues1[1] == maxVal1 and newValues2[1] == maxVal2 then
					self.mowers[i].isTransport = true;
					--print("self.mowers["..i.."].isTransport = true");
				end;

				
				
			elseif self.mowers[i].setUnfold == true and self.mowers[i].isWorking == false then
				self.mowers[i].isTransport = false;
				
				local rx1, ry1, rz1 = getRotation(self.componentJoints[self.mowers[i].idx1].jointNode);
				local newValues1 = {};
				if i == 2 then
					newValues1 = Utils.getMovedLimitedValues({rz1}, {maxAngle}, {maxVal1}, 1, 20000, dt, false);
				else
					newValues1 = Utils.getMovedLimitedValues({rz1}, {maxVal1}, {maxAngle}, 1, 20000, dt, true);
				end;
				
				setRotation(self.componentJoints[self.mowers[i].idx1].jointNode, rx1, ry1, newValues1[1]);
				setRotation(self.componentJoints[self.mowers[i].idx2].jointNode, rx1, ry1, newValues1[1]);
				if self.isServer then
					setJointFrame(self.componentJoints[self.mowers[i].idx1].jointIndex, 0, self.componentJoints[self.mowers[i].idx1].jointNode);
					setJointFrame(self.componentJoints[self.mowers[i].idx2].jointIndex, 0, self.componentJoints[self.mowers[i].idx2].jointNode);
				end;
				
				local rx2, ry2, rz2 = getRotation(self.mowers[i].rotationNode);
				local newValues2 = {};
				if i == 2 then
					newValues2 = Utils.getMovedLimitedValues({ry2}, {maxVal2}, {0}, 1, 20000, dt, true);
				else
					newValues2 = Utils.getMovedLimitedValues({ry2}, {0}, {maxVal2},  1, 20000, dt, false);
				end;
				
				setRotation(self.mowers[i].rotationNode, rx2, newValues2[1], rz2);
				if self.isServer then
					setJointFrame(self.componentJoints[self.mowers[i].idx3].jointIndex, 0, self.componentJoints[self.mowers[i].idx3].jointNode);
				end;
				
				if newValues1[1] == maxAngle and newValues2[1] == Utils.degToRad(0) then
					self.mowers[i].isWorking = true;
					--print("self.mowers["..i.."].isWorking = true");
				end;
				
			end;
		end;	
	end;
		
	
	--#### update hydraulics --------------------------------------------------
	for i, cylinder in pairs(self.cylinders) do
		local ax, ay, az = getWorldTranslation(cylinder.girl);
		local bx, by, bz = getWorldTranslation(cylinder.target);
		local x, y, z = worldDirectionToLocal(getParent(cylinder.girl), bx-ax, by-ay, bz-az);
		
		local upX, upY, upZ = 0,1,0;
		if math.abs(y) > 0.99*Utils.vector3Length(x, y, z) then
			-- direction and up is parallel
			upY = 0;
			if y > 0 then
				upZ = 1;
			else
				upZ = -1;
			end;
		end;
		
		setDirection(cylinder.girl, x, y, z, upX, upY, upZ);
		local distance = Utils.vector3Length(ax-bx, ay-by, az-bz);
		setTranslation(cylinder.boy, 0, 0, distance - cylinder.length2);			
	end;
	
	--## cabine control
	if self.updateCabineHUD then
		for i=1,4 do 
			setVisibility(self.cabineHUD[i].node1, self.mowers[i].setUnfold);
			setVisibility(self.cabineHUD[i].node2, self.mowers[i].setDown);
			setVisibility(self.cabineHUD[i].node3, self.mowers[i].setTurnedOn);
			setVisibility(self.cabineHUD[i].node4, self.mowers[i].setSwathOn);
		end;
		self.updateCabineHUD = false;
	end;	
	
	--##
	local cnt = 0;
	for i=1,3 do 
		if self.mowers[i].setTurnedOn == true then
			cnt = cnt + 1;
		end;	
	end;
	--self:setVehicleRpmUp(cnt);
	
end;

function KroneBigM500:draw()	
	
	if self.wasToFastForMowing then
		g_currentMission:addWarning(g_i18n:getText("Dont_drive_to_fast") .. "\n" .. string.format(g_i18n:getText("Cruise_control_levelN"), "2", InputBinding.getKeyNamesOfDigitalAction(InputBinding.SPEED_LEVEL2)), 0.07+0.022, 0.019+0.029);
	end;

	--if self.wasToFastForFolding then
	--	g_currentMission:addWarning("You are too fast for folding, please slow down!"); --g_i18n:getText("Dont_drive_to_fast") .. "\n" .. string.format(g_i18n:getText("Cruise_control_levelN"), "1", InputBinding.getKeyNamesOfDigitalAction(InputBinding.SPEED_LEVEL1)), 0.07+0.022, 0.019+0.029);
	--end;
	
	local xpos = 0.75;
	local ypos = 0.41; -- + 0.08;
	
	if self.hud.displayHUD == 1 then
		self.hud.overlay:render();
		
		--renderText(xpos, ypos+0.03, 0.02, g_i18n:getText("BIGM500_HUD"));
		renderText(xpos, ypos+0.03, 0.02, string.format(g_i18n:getText("BIGM500_HUDs"), InputBinding.getKeyNamesOfDigitalAction(InputBinding.BIGM500_HUD)));
		renderText(xpos, ypos, 0.02, g_i18n:getText("BIGM500_HUDtext1"));
		renderText(xpos, ypos-0.002, 0.02, "__________________________");
		renderText(xpos, ypos-0.02, 0.02, string.format(g_i18n:getText("BIGM500_SELECTs"), 
			InputBinding.getKeyNamesOfDigitalAction(InputBinding.BIGM500_SELECT), self.mowerTypes[self.activeMowerIdx]));
		renderText(xpos, ypos-0.04, 0.02, string.format(g_i18n:getText("BIGM500_UPDOWNs"), InputBinding.getKeyNamesOfDigitalAction(InputBinding.BIGM500_UPDOWN)));
		renderText(xpos, ypos-0.06, 0.02, string.format(g_i18n:getText("BIGM500_ONOFFs"), InputBinding.getKeyNamesOfDigitalAction(InputBinding.BIGM500_ONOFF)));
		renderText(xpos, ypos-0.08, 0.02, string.format(g_i18n:getText("BIGM500_SWATHs"), InputBinding.getKeyNamesOfDigitalAction(InputBinding.BIGM500_SWATH)));
		renderText(xpos, ypos-0.10, 0.02, string.format(g_i18n:getText("BIGM500_FOLDs"), InputBinding.getKeyNamesOfDigitalAction(InputBinding.BIGM500_FOLD)));
		
		renderText(xpos, ypos-0.13, 0.02, g_i18n:getText("BIGM500_HUDtext2"));		
		renderText(xpos, ypos-0.132, 0.02, "__________________________");		
		renderText(xpos, ypos-0.15, 0.02, string.format(g_i18n:getText("BIGM500_LEFTs"), InputBinding.getKeyNamesOfDigitalAction(InputBinding.BIGM500_LEFT)));		
		renderText(xpos, ypos-0.17, 0.02, string.format(g_i18n:getText("BIGM500_FRONTs"), InputBinding.getKeyNamesOfDigitalAction(InputBinding.BIGM500_FRONT)));		
		renderText(xpos, ypos-0.19, 0.02, string.format(g_i18n:getText("BIGM500_RIGHTs"), InputBinding.getKeyNamesOfDigitalAction(InputBinding.BIGM500_RIGHT)));		
	else
		renderText(xpos, ypos-0.17, 0.02, string.format(g_i18n:getText("BIGM500_HUDs"), InputBinding.getKeyNamesOfDigitalAction(InputBinding.BIGM500_HUD)));
		renderText(xpos, ypos-0.19, 0.02, string.format(g_i18n:getText("BIGM500_SELECTs"), 
			InputBinding.getKeyNamesOfDigitalAction(InputBinding.BIGM500_SELECT), self.mowerTypes[self.activeMowerIdx]));
	end;

end;

function KroneBigM500:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	-- saving just the boolean variables i not enough - ?!
	if not resetVehicles then
		for i=1,4 do 
			self.mowers[i].setUnfold = Utils.getNoNil(getXMLBool(xmlFile,key..string.format("#mowers%d.setUnfold",i)),false);
			self.mowers[i].setDown = Utils.getNoNil(getXMLBool(xmlFile,key..string.format("#mowers%d.setDown",i)),false);
			self.mowers[i].setTurnedOn = Utils.getNoNil(getXMLBool(xmlFile,key..string.format("#mowers%d.setTurnedOn",i)),false);
			self.mowers[i].setSwathOn = Utils.getNoNil(getXMLBool(xmlFile,key..string.format("#mowers%d.setSwathOn",i)),false);
			self.mowers[i].isTransport = Utils.getNoNil(getXMLBool(xmlFile,key..string.format("#mowers%d.isTransport",i)),false);
			self.mowers[i].isWorking = Utils.getNoNil(getXMLBool(xmlFile,key..string.format("#mowers%d.isWorking",i)),false);
			self.mowers[i].isUp = Utils.getNoNil(getXMLBool(xmlFile,key..string.format("#mowers%d.isUp",i)),false);
			self.mowers[i].isDown = Utils.getNoNil(getXMLBool(xmlFile,key..string.format("#mowers%d.isDown",i)),false);
		end;
		self.activeMowerIdx = Utils.getNoNil(getXMLInt(xmlFile,key.."#activeMowerIdx"),1);
		
		for i=2,3 do
			local rx = Utils.getNoNil(getXMLFloat(xmlFile,key..string.format("#mowers%d.rotationNode.x",i)), 0.0);
			local ry = Utils.getNoNil(getXMLFloat(xmlFile,key..string.format("#mowers%d.rotationNode.y",i)), 0.0);
			local rz = Utils.getNoNil(getXMLFloat(xmlFile,key..string.format("#mowers%d.rotationNode.z",i)), 0.0);
			setRotation(self.mowers[i].rotationNode, rx, ry, rz);
		end;
		
		local r1 = Utils.getNoNil(getXMLFloat(xmlFile,key..string.format("#mowers%d.jointRotLimit",1)), 0.0);
		local r2 = Utils.getNoNil(getXMLFloat(xmlFile,key..string.format("#mowers%d.jointRotLimit2",1)), 0.0);
		setJointRotationLimit(self.componentJoints[self.mowers[1].idx1].jointIndex, 2, true, -r1, r1);
		setJointRotationLimit(self.componentJoints[self.mowers[1].idx2].jointIndex, 2, true, -r2, r2);	
		for i=2,3 do
			local r1 = Utils.getNoNil(getXMLFloat(xmlFile,key..string.format("#mowers%d.jointRotLimits.x",i)), 0.0);
			local r2 = Utils.getNoNil(getXMLFloat(xmlFile,key..string.format("#mowers%d.jointRotLimits.y",i)), 0.0);
			local r3 = Utils.getNoNil(getXMLFloat(xmlFile,key..string.format("#mowers%d.jointRotLimits.z",i)), 0.0);
			setJointRotationLimit(self.componentJoints[self.mowers[i].idx1].jointIndex, 2, true, -r1, r1);
			setJointRotationLimit(self.componentJoints[self.mowers[i].idx2].jointIndex, 2, true, -r1, r1);		
			setJointRotationLimit(self.componentJoints[self.mowers[i].idx3].jointIndex, 0, true, -r2, r2);	
			setJointRotationLimit(self.componentJoints[self.mowers[i].idx3].jointIndex, 2, true, -r3, r3);				
		end;

		for i=1,10 do 
			local rx = Utils.getNoNil(getXMLFloat(xmlFile,key..string.format("#compJoint%d.x",i)), 0.0);
			local ry = Utils.getNoNil(getXMLFloat(xmlFile,key..string.format("#compJoint%d.y",i)), 0.0);
			local rz = Utils.getNoNil(getXMLFloat(xmlFile,key..string.format("#compJoint%d.z",i)), 0.0);
			setRotation(self.componentJoints[i].jointNode, rx, ry, rz);
			setJointFrame(self.componentJoints[i].jointIndex, 0, self.componentJoints[i].jointNode);
		end; 
		
	end;
    return BaseMission.VEHICLE_LOAD_OK; 
end;


function KroneBigM500:getSaveAttributesAndNodes(nodeIdent)

	local attributes = ' ';
	for i=1,4 do
		local mystring = 
		' mowers'..i..'.setUnfold="'..tostring(self.mowers[i].setUnfold)..'"'..
		' mowers'..i..'.setDown="'..tostring(self.mowers[i].setDown)..'"'..
		' mowers'..i..'.setTurnedOn="'..tostring(self.mowers[i].setTurnedOn)..'"'..
		' mowers'..i..'.setSwathOn="'..tostring(self.mowers[i].setSwathOn)..'"'..
		' mowers'..i..'.isTransport="'..tostring(self.mowers[i].isTransport)..'"'..
		' mowers'..i..'.isWorking="'..tostring(self.mowers[i].isWorking)..'"'..
		' mowers'..i..'.isUp="'..tostring(self.mowers[i].isUp)..'"'..
		' mowers'..i..'.isDown="'..tostring(self.mowers[i].isDown)..'"';
		attributes = attributes .. mystring;
	end;
	local mystring = ' self.activeMowerIdx="'..tostring(self.activeMowerIdx)..'"';
	attributes = attributes .. mystring;
	
	for i=2,3 do
		local rx, ry, rz = getRotation(self.mowers[i].rotationNode);
		local mystring = 
		' mowers'..i..'.rotationNode.x="'..tostring(rx)..'"'..
		' mowers'..i..'.rotationNode.y="'..tostring(ry)..'"'..
		' mowers'..i..'.rotationNode.z="'..tostring(rz)..'"';
		attributes = attributes .. mystring;
	end;
	
	local mystring = ' mowers1.jointRotLimit="'..tostring(self.mowers[1].jointRotLimit)..'"'..
		' mowers1.jointRotLimit2="'..tostring(self.mowers[1].jointRotLimit2)..'"';
	attributes = attributes .. mystring;
	
	for i=2,3 do
		local mystring = ' mowers'..i..'.jointRotLimits.x="'..tostring(self.mowers[i].jointRotLimits[1])..'"'..
			' mowers'..i..'.jointRotLimits.y="'..tostring(self.mowers[i].jointRotLimits[2])..'"'..
			' mowers'..i..'.jointRotLimits.z="'..tostring(self.mowers[i].jointRotLimits[3])..'"';
		attributes = attributes .. mystring;
	end
	
	
	for i=1,10 do
		local rx, ry, rz = getRotation(self.componentJoints[i].jointNode);
		local mystring = 
		' compJoint'..i..'.x="'..tostring(rx)..'"'..
		' compJoint'..i..'.y="'..tostring(ry)..'"'..
		' compJoint'..i..'.z="'..tostring(rz)..'"';
		attributes = attributes .. mystring;
	end;
	
    local node = nil;
    return attributes, node;

end;

function KroneBigM500:onLeave()
	if self.deactivateOnLeave then
        self:onDeactivate(self);
    else
        self:onDeactivateSounds(self);
    end;
end;

function KroneBigM500:onEnter()
end;

function KroneBigM500:onDeactivate()
	self:setVehicleRpmUp(0);
	stopSample(self.mowerSound);
	stopSample(self.windrowerSound);
	for i=1,3 do
		self.mowerSoundEnabled[i] = false;
		self.windrowerSoundEnabled[i] = false;
	end;
end;

function KroneBigM500:setActiveMower(idx, noEventSend)

	SetActiveMowerEvent.sendEvent(self, idx, noEventSend);

	self.activeMowerIdx = idx;
	if self.activeMowerIdx == 4 then
		local setUnfold=0;
		local setDown=0;
		local setTurnedOn=0; 
		local setSwathOn=0;
		local isUp=0;
		local isWorking=0;
		for i=1, 3 do 
			if self.mowers[i].setUnfold then
				setUnfold = setUnfold + 1;
			end;
			if self.mowers[i].setDown then
				setDown = setDown + 1;
			end;
			if self.mowers[i].setTurnedOn then
				setTurnedOn = setTurnedOn + 1;
			end;
			if self.mowers[i].setSwathOn then
				setSwathOn = setSwathOn + 1;
			end;
		end;
		if setUnfold > 0 then
			self.mowers[self.activeMowerIdx].setUnfold = true;
		else
			self.mowers[self.activeMowerIdx].setUnfold = false;
		end;
		if setDown > 0 then
			self.mowers[self.activeMowerIdx].setDown = true;
		else
			self.mowers[self.activeMowerIdx].setDown = false;
		end;
		if setTurnedOn > 0 then
			self.mowers[self.activeMowerIdx].setTurnedOn = true;
		else
			self.mowers[self.activeMowerIdx].setTurnedOn = false;
		end;
		if setSwathOn > 0 then
			self.mowers[self.activeMowerIdx].setSwathOn = true;
		else
			self.mowers[self.activeMowerIdx].setSwathOn = false;
		end;
	end;
	self.updateCabineHUD = true;
end;

function KroneBigM500:setMowerOptions(setUnfold, setDown, setTurnedOn, setSwathOn, noEventSend)

	SetMowerOptionsEvent.sendEvent(self, setUnfold, setDown, setTurnedOn, setSwathOn, noEventSend);

	--print("setUnfold="..tostring(setUnfold).." setDown="..tostring(setDown).." setTurnedOn="..tostring(setTurnedOn).." setSwathOn="..tostring(setSwathOn));
	self.mowerStateChanged = true;
	if self.activeMowerIdx ~= 4 then
		self.mowers[self.activeMowerIdx].setUnfold = setUnfold;
		self.mowers[self.activeMowerIdx].setDown = setDown;
		self.mowers[self.activeMowerIdx].setTurnedOn = setTurnedOn;
		self.mowers[self.activeMowerIdx].setSwathOn = setSwathOn;
	else
		for i=1, 4 do 
			if self.mowers[4].setUnfold ~= setUnfold then
				self.mowers[i].setUnfold = setUnfold;
			end;
			if self.mowers[4].setDown ~= setDown then
				self.mowers[i].setDown = setDown;
			end;
			if self.mowers[4].setTurnedOn ~= setTurnedOn then
				self.mowers[i].setTurnedOn = setTurnedOn;
			end;
			if self.mowers[4].setSwathOn ~= setSwathOn then
				self.mowers[i].setSwathOn = setSwathOn;
			end;
		end;
	end;
	
	self.updateCabineHUD = true;
end;

function KroneBigM500:setBeaconLightsVisibility(visibility, noEventSend)
	for i, j in pairs(self.beacons) do
		setVisibility(j, visibility);
	end;
end;

function KroneBigM500:setCuttersState(validCutters, noEventSend)
	--self.currentFruitTypesSend = fruitTypes;
	--self.currentAreasSend = areas;
	--self.currentUpdate = true;
	local check = false;
	for i=1,4 do
		if validCutters[i] ~= self.psStateSend[i] then
			check = true;
		end;
	end;
	if check == true then
		SetCuttersStateEvent.sendEvent(self, validCutters, noEventSend);
		self.doPSstateUpdate = true;
		for i=1,4 do
			self.psStateSend[i] = validCutters[i];
		end;		
	end;
end;

function KroneBigM500:setVehicleRpmUp(cnt)
	self.motor.minRpm = 0 - cnt*180;
end;

